#ifndef FrameBuffer_h
#define FrameBuffer_h

#include "UtilsGL.h"
#include "Texture.h"
#include "RenderBuffer.h"
#include <map>

class FrameBuffer {
protected:
	int mode;

	GLuint frameBufferID;

	std::map<GLuint, Texture *> attachedTextures;

	std::map<GLuint, RenderBuffer *> attachedRenderBuffer;

	//Framebuffer errors
	void checkError(){
		GLenum status;                                            
        status = glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT); 
        switch(status) {                                          
          case GL_FRAMEBUFFER_COMPLETE_EXT:                       
            break;                                                
          case GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT:                  
			  std::cerr<<"FrameBuffer: "<<"Texture incomplete attachment\n";                
            break;                                         
		  case GL_INVALID_FRAMEBUFFER_OPERATION_EXT :
			std::cerr<<"FrameBuffer: "<<"GL_INVALID_FRAMEBUFFER_OPERATION_EXT\n";                
            break;
		  case GL_FRAMEBUFFER_UNSUPPORTED_EXT :
			std::cerr<<"FrameBuffer: "<<"GL_FRAMEBUFFER_UNSUPPORTED_EXT\n";                
            break;

          default:    
			  std::cerr<<"FrameBuffer: "<<"RenderBuffer error\n";
			  
            /* programming error; will fail on all hardware */                               
        }
	}
public:
	enum FBModes{FB_MODE_WINDOW, FB_MODE_FBO, FB_MODE_FBO_COPY};

	FrameBuffer(int m=FB_MODE_FBO){
		this->mode=m;

		frameBufferID=0;

		if(this->mode==FB_MODE_FBO || this->mode==FB_MODE_FBO_COPY){
			glGenFramebuffersEXT( 1, &(this->frameBufferID) );
			checkError();
		}
	}

	~FrameBuffer(){
		glDeleteFramebuffersEXT(1, &(this->frameBufferID));
	}


	void attachRenderBuffer(GLuint attachPoint, RenderBuffer *rb){
		attachedRenderBuffer[attachPoint]=rb;

		rb->attachToFramebuffer(this->frameBufferID, attachPoint);
	}

	void detachRenderBuffer(GLuint attachPoint){
		attachedRenderBuffer.erase(attachPoint);
	}

	void attachTexture(GLuint attachPoint, Texture *t, int zoffset=0){
		attachedTextures[attachPoint]=t;

		if(this->mode==FB_MODE_FBO){
			glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, frameBufferID );

			if(t->getTarget()==GL_TEXTURE_3D){
				if(zoffset<0)
					zoffset=0;
				glFramebufferTexture3DEXT(GL_FRAMEBUFFER_EXT, attachPoint, t->getTarget(), t->getTextureID(), 0, zoffset);
			}else
				glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, attachPoint, t->getTarget(), t->getTextureID(), 0);

			checkError();
			glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, 0 );
		}
	}

	void detachTexture(GLuint attachPoint){
		attachedTextures.erase(attachPoint);
	}

	void bind(){
		if(this->mode==FB_MODE_FBO || this->mode==FB_MODE_FBO_COPY)
			glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, frameBufferID );
		else{

		}
		//glViewport(0, 0, this->size.x, this->size.y);

	}

	void unbind(){
		if(this->mode==FB_MODE_FBO_COPY){
			std::map<GLuint, Texture *>::iterator textureIT;
			//Parcour de tt les textures attaches
			for(textureIT=this->attachedTextures.begin(); textureIT!=this->attachedTextures.end(); textureIT++){
				if(this->attachedRenderBuffer.find(textureIT->first)!=this->attachedRenderBuffer.end()){
					//TODO: glReadBuffer et lecture de depth  paramtrer
					this->attachedRenderBuffer[textureIT->first]->copyToTexture( textureIT->second );
				}
			}
		}

		if(this->mode==FB_MODE_FBO || this->mode==FB_MODE_FBO_COPY)
			glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, 0 );
	}
};

#endif