#ifndef RenderBuffer_h
#define RenderBuffer_h

#include "UtilsGL.h"
#include "Texture.h"

class RenderBuffer {
protected:
	GLuint bufferID;

	Vector3i size;
	GLuint intFormat;

	void checkError(){
		GLenum status;                                            
        status = glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT); 
        switch(status) {                                          
          case GL_FRAMEBUFFER_COMPLETE_EXT:                       
            break;                                                
          case GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT:                  
			  std::cerr<<"Texture incomplete attachment\n";                
            break;                                         
		  case GL_INVALID_FRAMEBUFFER_OPERATION_EXT :
			std::cerr<<"GL_INVALID_FRAMEBUFFER_OPERATION_EXT\n";                
            break;
		  case GL_FRAMEBUFFER_UNSUPPORTED_EXT :
			std::cerr<<"GL_FRAMEBUFFER_UNSUPPORTED_EXT\n";                
            break;

          default:    
			  std::cerr<<"RenderBuffer error\n";
			  
            /* programming error; will fail on all hardware */                               
        }
	}

public:

	RenderBuffer(const Vector3i &size, GLuint iformat){
		this->size=size;
		this->intFormat=iformat;

		glGenRenderbuffersEXT( 1, &(this->bufferID));

		glBindRenderbufferEXT( GL_RENDERBUFFER_EXT, this->bufferID );
		glRenderbufferStorageEXT( GL_RENDERBUFFER_EXT, this->intFormat, this->size.x, this->size.y );
		glBindRenderbufferEXT( GL_RENDERBUFFER_EXT, 0 );

		checkError();
	}

	~RenderBuffer(){
		glDeleteRenderbuffersEXT(1, &(this->bufferID));
	}

	GLuint getBufferID(){
		return this->bufferID;
	}

	void attachToFramebuffer(GLuint frameBufferID, GLuint attachPoint){
		glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, frameBufferID );
		glFramebufferRenderbufferEXT( GL_FRAMEBUFFER_EXT, attachPoint, GL_RENDERBUFFER_EXT, this->bufferID );
		glBindFramebufferEXT( GL_FRAMEBUFFER_EXT, 0 );
		checkError();
	}

	void copyToTexture(Texture *t){
		t->setFromFramebuffer();
	}


};
#endif