#include "ShaderObjectCg.h"

#include <iostream>
#include <string>
#include <string.h>

#include <Cg/cg.h>
#include <Cg/cgGL.h>


CGcontext ShaderObjectCg::context=NULL;




ShaderObjectCg::~ShaderObjectCg(){
  
}


void ShaderObjectCg::checkCgError(){
  CGerror err = cgGetError();
  
  if (err != CG_NO_ERROR){
	  std::cout<<this->shaderSourceEntry<<" :\n"<<cgGetErrorString(err)<<"\n"<<cgGetLastListing(this->context)<<"\n";
	  int dd;
	  std::cin>>dd;
	  //exit(1);
  }
}

void ShaderObjectCg::compile(){
  if(this->isCompiled)
	return;

  if(!this->isPrecompiled)
    this->precompile();


  if(this->program){
    cgCompileProgram(this->program);
    checkCgError();
  
    cgGLLoadProgram(this->program);
    checkCgError();

	this->isCompiled=true;
  }
}


void ShaderObjectCg::precompile(){

  if(this->isPrecompiled)
    return;


  if(this->context==NULL){
    this->context=cgCreateContext();
  }

  checkCgError();


  char arg0[]="-fastprecision";
  char arg1[]="-fastmath";
  char arg2[]="-posinv";
  char arg3[]="-unroll";
  char arg4[]="none";
  char *args[]={arg1, NULL, NULL, NULL};

  if(this->profile==CG_PROFILE_FP40){
	args[1]=arg3;
	args[2]=arg4;
  }

  cgSetAutoCompile(this->context, CG_COMPILE_MANUAL);

  if(this->shaderSource.size()>0){
    if(this->positionInvariant)
      args[0]=arg2;

    if( !cgGLIsProfileSupported(this->profile) )
		this->profile=cgGLGetLatestProfile(this->profileType);
    

    // Cree le programme
	this->program = cgCreateProgram(this->context, CG_SOURCE, this->shaderSource.c_str(), this->profile, this->shaderSourceEntry.c_str(), (const char **)args);
    checkCgError();

  }

  this->isPrecompiled=true;

}

void ShaderObjectCg::bind(){
  this->compile();

  if(this->program){
    cgGLBindProgram( this->program );
	checkCgError();
    cgGLEnableProfile( this->profile );
    checkCgError();
  }

}

void ShaderObjectCg::unbind(){
  if(this->program){
    cgGLUnbindProgram( this->profile );  
    cgGLDisableProfile( this->profile );
  }

}




unsigned int ShaderObjectCg::getParamLocation(const char *name){
    this->compile();

    int res;
    
    res=(int)cgGetNamedParameter(this->program, name);
    if(res==0){
      std::cerr<<"No parameter "<<name<<" found in shader";
      //exit(1);
    }

    return (res);
}

void ShaderObjectCg::setParam(unsigned int pname, float val){
	cgGLSetParameter1f( (CGparameter)pname, val );
}
void ShaderObjectCg::setParam(unsigned int pname, float val0, float val1, float val2){
	cgGLSetParameter3f( (CGparameter)pname, val0, val1, val2 );
}
void ShaderObjectCg::setParam(unsigned int pname, Vector3f &val){
	cgGLSetParameter3fv( (CGparameter)pname, (float*)&val.x );
}
void ShaderObjectCg::setParam(unsigned int pname, Vector4f &val){
	cgGLSetParameter4fv( (CGparameter)pname, (float*)&val.x );
}

void ShaderObjectCg::setParam(unsigned int pname, int nb, float *vals){
	switch(nb){
		case 1:
			cgGLSetParameter1fv( (CGparameter)pname, vals );
			break;
		case 2:
			cgGLSetParameter2fv( (CGparameter)pname, vals );
			break;
		case 3:
			cgGLSetParameter3fv( (CGparameter)pname, vals );
			break;
		case 4:
			cgGLSetParameter4fv( (CGparameter)pname, vals );
			break;
	}

}

void ShaderObjectCg::setSamplerParam(unsigned int pname, GLuint texid){
    this->compile();
    cgGLSetTextureParameter((CGparameter)pname, texid);
	cgGLEnableTextureParameter((CGparameter) pname);
    checkCgError();
}

void ShaderObjectCg::enableSamplerParam(unsigned int pname){
    //Bind();
    cgGLEnableTextureParameter((CGparameter) pname);
    checkCgError();
    //Unbind();
}
void ShaderObjectCg::disableSamplerParam(unsigned int pname){
    //Bind();
    cgGLDisableTextureParameter((CGparameter) pname);
    checkCgError();
    //Unbind();
}
