#ifndef ShaderObjectCg_h
#define ShaderObjectCg_h

#include "ShaderObject.h"

#include <Cg/cg.h>
#include <Cg/cgGL.h>


/** Manipulation de shaders ecris en Cg. */
class ShaderObjectCg : public ShaderObject{
protected:
  static CGcontext context;

  CGprogram program;

  CGGLenum profileType;
  CGprofile profile;

  CGerror CgError;

  void checkCgError();

  virtual bool isCorrectFileName(const char *str){
    return ShaderObject::isFileName(str, ".cg");
  }

  void compile();
  void precompile();

  bool isPrecompiled;
  bool isCompiled;
public:
  

  ShaderObjectCg(const char *src, ProfileType pt=VERTEX_PROGRAM, const char *entryfunc="main", const char *options=""){
	if(src)
		this->setShaderSource(src);

	shaderSourceEntry=entryfunc;


	this->program=NULL;
	this->profileType=(CGGLenum)(CG_GL_VERTEX+pt);
	this->profile=cgGLGetLatestProfile((CGGLenum)(CG_GL_VERTEX+pt));

	this->isCompiled=false;
	this->isPrecompiled=false;

  }

  virtual ~ShaderObjectCg();

  void setProfile(CGprofile p){
	  this->profile=p;
  }
  void setProfileType(CGGLenum pt){
	  this->profileType=pt;
  }

  void bind();
  void unbind();


  //Recup des ids params
  unsigned int getParamLocation(const char *name);
  
  //Passage de parametres//
  void setParam(unsigned int pname, float val);
  void setParam(unsigned int pname, float val0, float val1, float val2);
  void setParam(unsigned int pname, Vector3f &val);
  void setParam(unsigned int pname, Vector4f &val);
  void setParam(unsigned int pname, int nb, float *vals);

  void setSamplerParam(unsigned int pname, GLuint texid);
  void enableSamplerParam(unsigned int pname);
  void disableSamplerParam(unsigned int pname);
  

  void setStateMatrixParam(unsigned int pname, GLenum state, GLenum transform){
	cgGLSetStateMatrixParameter((CGparameter)pname, (CGGLenum)state, (CGGLenum)transform);
  }

  void setParam(const char *pname, Vector3f &val){
	cgGLSetParameter3fv( cgGetNamedParameter(this->program, pname), (float*)&val.x );
  }
};


#endif
