#include "GLAppParallelSort.h"
#include "l3ds.h"
#include "TGAManip.h"

#ifdef WIN32
#include <GL/wglew.h>
#endif


/////Fluid/////////
/* macros */
#define IX(i,j) ((i)+(N+2)*(j))

/* external definitions (from solver.c) */
extern void dens_step ( int N, float * x, float * x0, float * u, float * v, float diff, float dt );
extern void vel_step ( int N, float * u, float * v, float * u0, float * v0, float visc, float dt );
static float * u, * v, * u_prev, * v_prev;
static float dt= 0.01f, diff=0, visc=0.001;

int perturbcount=0;

//////////////////

GLAppParallelSort::GLAppParallelSort(const char *title, int w, int h, int flags):
						GLApp(title, w, h, flags), viewOrient(0,0,0){
	
	mouseButtons[0]=mouseButtons[1]=mouseButtons[2]=0;
	pause=false;
	
	this->wireframe=false;
}

void GLAppParallelSort::keyboard(unsigned char key) {
  Vector3f col;

  float ddens;

  switch (key) {
	case 'q':
		exit(0);
		break;
	case 'w':
		wireframe=!wireframe;
		break;
	case 'p':
		pause=!pause;
		this->dispText(2000, std::string("Pause: ")+this->pause, Vector2i(200,20), Vector4f(1.0f, 1.0f, 0.0f, 1.0f));
		break;
	case 'r':
		reset=true;
		break;
	case 'a':
		perturb=true;
		break;
  }

}


void GLAppParallelSort::mouse(int button, int state, int x, int y){
	oldMousePos.x=x;
	oldMousePos.y=y;

	switch(button){
	case GLUT_LEFT_BUTTON:
		this->mouseButtons[0] = !state;
		break;
	case GLUT_RIGHT_BUTTON:
		this->mouseButtons[2] = !state;
		break;
	case GLUT_MIDDLE_BUTTON:
		this->mouseButtons[1] = !state;
		break;
	}
}


void GLAppParallelSort::mouseMove(int x, int y){

	float rx =(float)(x-oldMousePos.x);
	float ry =(float)(y-oldMousePos.y);

	Vector2i wMove=Vector2i( (x-oldMousePos.x)/1, 0-(y-oldMousePos.y)/1);

	if(mouseButtons[2]){
		this->viewOrient.x+=ry/5.0f;
		this->viewOrient.y+=rx/5.0f;
	}
	
	if(this->viewOrient.x>90 && perturbcount<=0){
		perturb=true;
	}
	oldMousePos.x = x;
	oldMousePos.y = y;
	
}

void GLAppParallelSort::mousePassiveMove(int x, int y){

}

void GLAppParallelSort::perspectiveGL( GLdouble fovY, GLdouble aspect, GLdouble zNear, GLdouble zFar ) {

	const GLdouble pi = 3.1415926535897932384626433832795;

	GLdouble fW, fH;

	fH = tan( fovY / 360 * pi ) * zNear;
		
	fW = fH * aspect;

	glFrustum( -fW, fW, -fH, fH, zNear, zFar );

}


void GLAppParallelSort::reshape(int w, int h) { 
  glMatrixMode(GL_PROJECTION );
  glLoadIdentity();
  perspectiveGL(60.0, (float) w / (float) h, 0.10, 1000.0);

  glMatrixMode(GL_MODELVIEW);
  glViewport(0, 0, w, h);

}


void GLAppParallelSort::idle() { 


	glViewport(0, 0, this->windowSize.x, this->windowSize.y);
	glClearColor(0.0,0.0,0.0,0.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

 


	if(reset){

		for(int i=0; i<TSIZE; i++){
			data[0][i]=data[1][i]=(rand()%TSIZE)/((float)TSIZE);
		}
		
		reset=false;
	}
	if(perturb){
		for(int i=0; i<TSIZE; i++){
			data[0][i]=data[1][i]=(rand()%TSIZE)/((float)TSIZE);
		}

		for(int i=domSize.x; i<domSize.x*5; i++){
			v[i]=(rand()%1000)/((float)1.0f);
			u[i]=(rand()%1000)/((float)1000.0f);
		}
		perturbcount=100;
		perturb=false;
	}

	if(perturbcount>0){
		
		/*if(!(perturbcount%4)){
			for(int i=0; i<TSIZE; i++){
				data[!(sortLoop%2)][i]=data[!(sortLoop%2)][i]*0.5+ ( (rand()%3)/2.0f)*0.5;
			}
		}*/
		if(!(perturbcount%10)){
			/**for(int i=domSize.x; i<domSize.x*4; i++){
				data[0][i]=data[1][i]=((rand()%TSIZE)/((float)TSIZE)) * sin((double)i/10.0f);
			}*/

			/*for(int i=domSize.x; i<domSize.x*4-1; i++){
				data[!(sortLoop%2)][i]=(data[!(sortLoop%2)][i]+data[!(sortLoop%2)][i+1]+data[!(sortLoop%2)][i-domSize.x]+data[!(sortLoop%2)][i+domSize.x])/4.0f;
			}*/
		
			for(int i=domSize.x; i<domSize.x*2; i++)
				data[0][i]=data[1][i]=0;

			for(int i=domSize.x*2; i<domSize.x*3; i++)
				data[0][i]=data[1][i]=0.5;

			for(int i=domSize.x*3; i<domSize.x*5; i++)
				data[0][i]=data[1][i]=1.0;

			for(int i=TSIZE-domSize.x*2; i<TSIZE; i++)
				data[0][i]=data[1][i]=0;

			for(int i=TSIZE/2; i<TSIZE/2+domSize.x; i++)
				data[0][i]=data[1][i]=1.0;

			/*for(int i=domSize.x*5; i<domSize.x*6; i++)
				data[0][i]=data[1][i]=0.0;*/



		}

		perturbcount--;

		if(perturbcount<=0){
			this->pause=false;

			/*for(int i=0; i<TSIZE; i++){
				data[!(sortLoop%2)][i]=(data[!(sortLoop%2)][i]+(rand()%TSIZE)/((float)TSIZE))/2.0f;
			}*/
		}else
			this->pause=true;

		vel_step (domSize.x-2, u, v, u_prev, v_prev, visc, dt );
		dens_step ( domSize.x-2, data[(sortLoop%2)], data[!(sortLoop%2)], u, v, diff, dt );

		sortLoop++;

	}


	if(!this->pause){

		//this->startTimer();
		
	  	triStep(TSIZE, this->sortLoop, data);
	/*	float *data2[2];
		data2[0]=data[0]+TSIZE/2;
		data2[1]=data[1]+TSIZE/2;
		triStep(TSIZE, this->sortLoop, data);*/
	
		
		sortLoop++;
		/*triStep(TSIZE, this->sortLoop, data);
		sortLoop++;*/

		//merge(TSIZE, data[sortLoop%2], data2[sortLoop%2], data[2]);
		
	

		

		/*for (int i=0 ; i<domSize.x*domSize.y ; i++ ) {
			u[i] = v[i] = 0.0f;
		}*/

		

		/*vel_step (domSize.x-2, u, v, u_prev, v_prev, visc, dt );
		dens_step ( domSize.x-2, data[!(sortLoop%2)], data[(sortLoop%2)], u, v, diff, dt );*/

	//	sortLoop++;

	}
		
	texSolution->setData(data[!(sortLoop%2)], GL_RED, GL_FLOAT);
	

	this->redisplay();
}

void GLAppParallelSort::dispProd(){
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_NORMAL_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);

	glMatrixMode(GL_MODELVIEW);

	glPushMatrix();
		glScalef(0.007,0.007,0.007);

		texSolution->bind();
		dispShader->bind();

		/*glDisable(GL_BLEND);
		glEnable(GL_DEPTH_TEST);
		glDepthMask(true);*/
		glEnable(GL_BLEND);
		glDisable(GL_DEPTH_TEST);
		glDepthMask(true);
		//glDisable(GL_CULL_FACE);


		glRotatef(90, 0,0,1);
		//glScalef(0.1,0.1,1);

		for(int k=0; k<20; k++){
			glPushMatrix();
			glScalef(k/20.0f,k/20.0f,1);
			for (uint i= 0; i<prod3DS.GetMeshCount(); i++)
			{
				LMesh &mesh = prod3DS.GetMesh(i);
				glTexCoordPointer(2, GL_FLOAT,0, &mesh.GetUV(0));
				glVertexPointer(4, GL_FLOAT, 0, &mesh.GetVertex(0));
				glNormalPointer(GL_FLOAT, 0, &mesh.GetNormal(0));
				glColorPointer(3, GL_FLOAT, 0, &mesh.GetBinormal(0));
				glDrawElements(GL_TRIANGLES, mesh.GetTriangleCount()*3, 
								GL_UNSIGNED_SHORT, &mesh.GetTriangle(0));

			}
			glPopMatrix();
		}

	

		dispShader->unbind();
		texSolution->unbind();
	glPopMatrix();

	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableClientState(GL_NORMAL_ARRAY);
	glDisableClientState(GL_COLOR_ARRAY);
}

void GLAppParallelSort::dispBecher(){
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_NORMAL_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);

	glMatrixMode(GL_MODELVIEW);

	glPushMatrix();
		glScalef(0.007,0.007,0.007);

		
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); //Back to front
		//glDisable(GL_DEPTH_WRITE);
		glDepthMask(false);


		texGrads->bind();
		becherShader->setSamplerParam(becherShader->getParamLocation("tex"), texGrads->getTextureID());
		becherShader->bind();
		for (uint i= 0; i<objet3DS.GetMeshCount(); i++)
		{
			LMesh &mesh = objet3DS.GetMesh(i);
			glTexCoordPointer(2, GL_FLOAT,0, &mesh.GetUV(0));
			glVertexPointer(4, GL_FLOAT, 0, &mesh.GetVertex(0));
			glNormalPointer(GL_FLOAT, 0, &mesh.GetNormal(0));
			glColorPointer(3, GL_FLOAT, 0, &mesh.GetBinormal(0));
			glDrawElements(GL_TRIANGLES, mesh.GetTriangleCount()*3, 
							GL_UNSIGNED_SHORT, &mesh.GetTriangle(0));

		}

		texGrads->unbind();
		becherShader->unbind();

	glPopMatrix();
	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableClientState(GL_NORMAL_ARRAY);
	glDisableClientState(GL_COLOR_ARRAY);

}

void GLAppParallelSort::display() {

	glDepthMask(GL_TRUE);
	glClearColor(0.0,0.0,0.0,0.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	glViewport(0, 0, this->windowSize.x, this->windowSize.y);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();


	/////////////
		/*GLfloat LightAmbient[]= { 0.01f, 0.01f, 0.01f, 1.0f };
		GLfloat LightDiffuse[]= { 0.1f, 0.1f, 0.1f, 1.0f };	
		GLfloat LightPosition[]= { 8.0f, 8.0f, 8.0f, 1.0f };
		glLightfv(GL_LIGHT1, GL_AMBIENT, LightAmbient);	
		glLightfv(GL_LIGHT1, GL_DIFFUSE, LightDiffuse);	
		//glLightfv(GL_LIGHT1, GL_SPECULAR, LightDiffuse);	
		glLightfv(GL_LIGHT1, GL_POSITION,LightPosition);	
		//glLightf(GL_LIGHT1, GL_LINEAR_ATTENUATION, 0.3);
		glEnable(GL_LIGHT1);
		glEnable(GL_LIGHTING);
		//glDisable(GL_CULL_FACE);*/
	/////////////




	glTranslatef(0,0,-2);
	//glTranslatef(this->viewPos.x, this->viewPos.y, this->viewPos.z);

	
	glRotatef(this->viewOrient.x, 1.0, 0.0, 0.0);
	glRotatef(this->viewOrient.y, 0.0, 1.0, 0.0);


	glEnable(GL_DEPTH_TEST);
	//glDepthMask(GL_TRUE);


	glPushMatrix();

		//glEnable(GL_BLEND);
		//glBlendFunc(GL_SRC_ALPHA, GL_ONE); //Back to front
		//glBlendFunc(GL_SRC_ALPHA_SATURATE, GL_ONE); //Front to back
		//glBlendFunc(GL_ONE_MINUS_DST_ALPHA, GL_DST_ALPHA);

		//glPolygonMode(GL_FRONT_AND_BACK , GL_LINE );
		
		/////////
		glRotatef(-90, 1,0,0);
		glTranslatef(0,0,-0.5);

		glPushMatrix();

			glTranslatef(-0.5,0,0);
			glEnable(GL_CULL_FACE);
			glCullFace(GL_FRONT);
			glColor4f(0.5,0.5,1,0.2);
			dispBecher();
			glCullFace(GL_BACK);

			dispProd();

			
			dispBecher();
		glPopMatrix();

		glPushMatrix();
			glTranslatef(0.5,0,0);
			glEnable(GL_CULL_FACE);
			glCullFace(GL_FRONT);
			glColor4f(1.0,0.5,0.5,0.2);
			dispBecher();
			glCullFace(GL_BACK);

			dispProd();

			
			dispBecher();
		glPopMatrix();

		
		//glPolygonMode(GL_FRONT_AND_BACK , GL_FILL );
	glPopMatrix();

	this->print("Tri Sequentiel", Vector2i(80*this->windowSize.x/500,450*this->windowSize.y/500), Vector4f(1.0f, 1.0f, 0.0f, 1.0f));
	this->print("Tri Parallele", Vector2i(this->windowSize.x-180*this->windowSize.x/500,450*this->windowSize.y/500), Vector4f(1.0f, 1.0f, 0.0f, 1.0f));



 //Buffer automatiquement swapp
}

void GLAppParallelSort::init(){
	glShadeModel (GL_SMOOTH);
	glEnable(GL_CULL_FACE);
	//glCullFace(GL_FRONT);

	glDisable(GL_DEPTH_TEST);
	glDisable(GL_LIGHTING);
	glDepthMask(true);

	glClearColor(0.0, 0.0, 0.0, 0.0);
	glClearDepth(1.0);


	this->showFPS(true);

	glewInit();

//Vsync off
#ifdef WIN32
	wglSwapIntervalEXT(0);
#endif

	///Init media path///
	this->mediapath=getenv("PS_MEDIA_PATH");

	////Init scene////
	objet3DS.LoadFile((this->mediapath + "/Becher3.3ds").c_str());
	prod3DS.LoadFile((this->mediapath +"/Produit2.3ds").c_str());

	domSize=Vector3i(128, 128, 0);
	
	TSIZE=domSize.x*domSize.y;
	sortLoop=0;
	reset=true;
	perturb=false;
	pause=true;
	/////////////////
	float *data0=new float[TSIZE];
	float *data1=new float[TSIZE];

	float *datares=new float[TSIZE];

	srand(time(0));

	for(int i=0; i<TSIZE; i++){
		data0[i]=(rand()%TSIZE)/((float)TSIZE);
		data1[i]=datares[i]=0;//data0[i];
	}
	////////////////

	texSolution=new Texture(GL_TEXTURE_2D, domSize, GL_RGBA8, GL_RED, GL_FLOAT, data0);
	texSolution->setFilterMode(GL_LINEAR);
	texSolution->setWrapMode(GL_MIRRORED_REPEAT);

	////////////////
	
	this->data[0]=data0;
	this->data[1]=data1;
	this->data[2]=datares;


	/*tri(TSIZE/2, data0, data1);
	tri(TSIZE/2, data0+TSIZE/2, data1+TSIZE/2);
	
	merge(TSIZE, data1, data1+TSIZE/2, datares);*/

	/*std::cout<<"res: ";
	for(int i=0; i<TSIZE; i++){
		std::cout<<datares[i]<<" ";
	}
	std::cout<<"\n";*/

	/*glEnable(GL_MULTISAMPLE_ARB);
	GLboolean tt;

	glGetBooleanv(GL_SAMPLE_BUFFERS_ARB, &tt);
	std::cout<<":"<<(int)tt<<":\n";*/


	dispShader=new ShaderObjectCg( (this->mediapath +"/Shaders/SortShader.cg").c_str(), ShaderObject::FRAGMENT_PROGRAM, "disp");
	becherShader=new ShaderObjectCg( (this->mediapath +"/Shaders/SortShader.cg").c_str(), ShaderObject::FRAGMENT_PROGRAM, "becher");



	tImageTGA img;
	LoadTGA((this->mediapath +"/Graduations2.tga").c_str(), &img);
	texGrads=new Texture(GL_TEXTURE_2D, Vector3i(img.sizeX, img.sizeY, 0), GL_RGBA8, GL_RGBA, GL_UNSIGNED_BYTE, img.data);
	texGrads->setFilterMode(GL_LINEAR);


	/////////Fluid////////
	u		= new float[domSize.x*domSize.y];
	v		= new float[domSize.x*domSize.y];
	u_prev		= new float[domSize.x*domSize.y];
	v_prev		= new float[domSize.x*domSize.y];

	for(int i=0; i<domSize.x*domSize.y; i++){
			u[i]=v[i]=u_prev[i]=v_prev[i]=0.0;
	}


}


void GLAppParallelSort::triKernel(int i, float *datain, float *dataout){

	if(datain[i]>datain[i+1]){
		dataout[i]=datain[i+1];
		dataout[i+1]=datain[i];
	}else{
		dataout[i]=datain[i];
		dataout[i+1]=datain[i+1];
	}

}

void GLAppParallelSort::tri(int size, float *datain, float *dataout){
	float *data[2];
	data[0]=datain;
	data[1]=dataout;

	for(int n=0; n<size+1; n++){
		/*std::cout<<"in: ";
		for(int i=0; i<size; i++)
			std::cout<<data[n%2][i]<<" ";
		std::cout<<"\n";*/

		for(int i=0; i<size/2 -(n%2); i++){
			triKernel(i*2+(n%2), data[n%2], data[!(n%2)]);
			if(n%2){
				data[0][0]=data[1][0];
				data[0][size-1]=data[1][size-1];
			}
		}

		//std::cout<<"out: ";
		/*for(int i=0; i<size; i++)
			std::cout<<data[!(n%2)][i]<<" ";
		std::cout<<"\n";*/
	}
}

void GLAppParallelSort::triStep(int size, int n, float *data[2]){

	for(int i=0; i<size/2 -(n%2); i++){
		triKernel(i*2+(n%2), data[n%2], data[!(n%2)]);
		if(n%2){
			data[0][0]=data[1][0];
			data[0][size-1]=data[1][size-1];
		}
	}

}

void GLAppParallelSort::merge(int size, float *datain0, float *datain1, float *dataout){

	int pos0=0, pos1=0;

	for(int i=0; i<size; i++){
		if( (datain0[pos0]<datain1[pos1] || pos1==size/2) && pos0!=size/2){
			dataout[i]=datain0[pos0];
			pos0++;
		}else{
			dataout[i]=datain1[pos1];
			pos1++;
		}
	} 
}
