#ifndef ShaderObject_h
#define ShaderObject_h

#include "UtilsGL.h"

#include <string>
#include <map>

class ShaderObject {
protected:
  /** Chargement d'un shader  partir d'un fichier.*/
  std::string loadSource(const char *name);
  
  /**Test nom de fichier correct.*/
  bool isFileName(const char *str, const char *ext);
  virtual bool isCorrectFileName(const char *str)=0;

 
  std::string shaderSource;
  std::string shaderSourceEntry;


  /**Options.*/
  bool positionInvariant;


  /**Constructors.*/
  ShaderObject(){
	  this->positionInvariant=false;
  }

  virtual ~ShaderObject(){}

public:
  enum ProfileType{VERTEX_PROGRAM=0, FRAGMENT_PROGRAM=1};

  /** Utilisation des fonctions fixes de transformation dans le vertex program.*/
  void setPositionInvariant(bool p){
		positionInvariant=p;
  }

  bool getPositionInvariant(){
	  return positionInvariant;
  }

  //////////////////////////////
  /** Recuperation de la source. */
  std::string &getShaderSource();

  /** Definition de la source. */
  void setShaderSource(const char *src, const char *entryfunc="main");
  /////////////////////////////

  /** Affectation du shader. */
  virtual void bind()=0;
  /** Desafectation du shader. */
  virtual void unbind()=0;


  /** Recuperation des ids de parametres  partir du nom. */
  virtual unsigned int getParamLocation(const char *name)=0;
  
  /** Passage de parametres. */
  virtual void setParam(unsigned int pname, float val)=0;
  virtual void setParam(unsigned int pname, float val0, float val1, float val2)=0;
  virtual void setParam(unsigned int pname, Vector3f &val)=0;
  virtual void setParam(unsigned int pname, Vector4f &val)=0;
  virtual void setParam(unsigned int pname, int nb, float *vals)=0;

  virtual void setSamplerParam(unsigned int pname, GLuint texid){}
  virtual void enableSamplerParam(unsigned int pname){}
  virtual void disableSamplerParam(unsigned int pname){}


  virtual void setStateMatrixParam(unsigned int pname, GLenum state, GLenum transform){}

  ////////////////////////////////////////////
  /** Passage de parametres par leur nom */
  void setParam(const char *pname, float val){
    setParam(getParamLocation(pname), val);
  }
  void setParam(const char *pname, float val0, float val1, float val2){
    setParam(getParamLocation(pname), val0, val1, val2);
  }

  void setParam(const char *pname, Vector3f &val){
    setParam(getParamLocation(pname), val);
  }
  void setParam(const char *pname, Vector4f &val){
    setParam(getParamLocation(pname), val);
  }

  void setParam(const char *pname, int nb, float *vals){
    setParam(getParamLocation(pname), nb, vals);
  }
  void setSamplerParam(const char *pname, GLuint texid){
	setSamplerParam(getParamLocation(pname), texid);
  }
  void enableSamplerParam(const char *pname){
	enableSamplerParam(getParamLocation(pname));
  }
  void disableSamplerParam(const char *pname){
	disableSamplerParam(getParamLocation(pname));
  }

  void setStateMatrixParam(const char *pname, GLenum state, GLenum transform){
	setStateMatrixParam(getParamLocation(pname), state, transform);
  }
  
  ////////////////////////////////////////////

};

#endif