#include "Texture.h"
#include "FrameBuffer.h"


void Texture::create(GLuint target, const Vector3i &size, GLuint iformat, GLuint eformat, GLuint t, void *data){
	if(this->texID!=0)
		glDeleteTextures(1, &(this->texID));

	glGenTextures(1, &(this->texID));
	this->type=t;
	this->target=target;
	this->size=size;

	this->intFormat=iformat;
	this->format=eformat;


	glBindTexture(target, this->texID);

	glTexParameteri(target, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexParameteri(target, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(target, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(target, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(target, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);

	if(target==GL_TEXTURE_3D){
		glTexImage3D( target, 0, intFormat, size.x, size.y, size.z, 0, format, type, data );
	}else if(target==GL_TEXTURE_CUBE_MAP){

	}else {
		glTexImage2D( target, 0, intFormat, size.x, size.y, 0, format, type, data );
	}

	glBindTexture(target, 0);
}

void Texture::setData(void *d, GLuint f, GLuint t, GLuint ta){
	if(f!=0)
		this->format=f;

	if(t!=0)
		this->type=t;

	GLuint target;

	//for cubes maps
	if(ta!=0)
		target=ta;
	else
		target=this->target;
		
	this->bind();

	if(target==GL_TEXTURE_3D){
		glTexImage3D( target, 0, intFormat, size.x, size.y, size.z, 0, format, type, d );
	}else{
		glTexImage2D( target, 0, intFormat, size.x, size.y, 0, format, type, d );
	}

	this->unbind();
}


Texture::Texture(GLuint target, const Vector3i &size, GLuint iformat, GLuint eformat, GLuint type, void *data){
	texID=0;
	this->create(target, size, iformat, eformat, type, data);
}
Texture::~Texture(){
	if(this->texID!=0)
		glDeleteTextures(1, &(this->texID));
}

Texture::Texture(const Texture &t){
	texID=0;

	unsigned char *buff=new unsigned char[t.getDataSize()];

	t.getData(buff, t.getType());

	this->create(t.getTarget(), t.getSize(), t.getInternalFormat(), t.getFormat(), t.getType(), buff);

	delete [] buff;
}

GLuint Texture::getTarget() const{
	return this->target;
}

GLuint Texture::getType() const{
	return this->type;
}

void Texture::setTarget(GLuint t){
	this->target = t;
}

GLuint Texture::getTextureID() const{
	return this->texID;
}

Vector3i Texture::getSize() const{
	return this->size;
}

void Texture::bind() const{
	glBindTexture(this->target, this->texID);
	glEnable(this->target);
}

void Texture::unbind() const{
	glBindTexture(this->target, 0);
	glDisable(this->target);
}

void Texture::setFromFramebuffer(){
	glBindTexture(this->target, this->texID);
	glCopyTexSubImage2D(this->target, 0, 0, 0, 0, 0, this->size.x, this->size.y);
//	glCopyTexImage2D(this->target, 0, this->intFormat, 0, 0, this->size.x, this->size.y, 0);
	glBindTexture(this->target, 0);

}

void Texture::setWrapMode(GLuint m){
	glBindTexture(target, this->texID);
	glTexParameteri(target, GL_TEXTURE_WRAP_S, m);
	glTexParameteri(target, GL_TEXTURE_WRAP_T, m);
	glTexParameteri(target, GL_TEXTURE_WRAP_R, m);
	glBindTexture(target, 0);
}

void Texture::setFilterMode(GLuint m){
	glBindTexture(target, this->texID);
	glTexParameteri(target, GL_TEXTURE_MIN_FILTER, m);
	glTexParameteri(target, GL_TEXTURE_MAG_FILTER, m);
	glBindTexture(target, 0);
}


GLuint Texture::getFormat() const{
	return this->format;
}

GLuint Texture::getInternalFormat() const{
	return this->intFormat;
}

int Texture::getDataSize() const{
	return getNumElem()*GLGetFormatSize(this->intFormat);
}

int Texture::getNumElem() const{
	if(this->size.y && this->size.z)
		return this->size.x*this->size.y*this->size.z;

	if(this->size.y)
		return this->size.x*this->size.y;

	return this->size.x;
}

void Texture::getData(DataBuffer *db, GLuint type) const{
	if(type==0)
		type=this->type;

	//Attention qd type different du type interne, verif des tailles

	if(db->getSize()<this->getDataSize()){
		std::cerr<<"Texture::getData -> Error: unmatching sizes\n";
		return;
	}

	this->bind();

	db->bind(GL_PIXEL_PACK_BUFFER_EXT);

	glGetTexImage(this->getTarget(), 0, this->getFormat(), type, db->use());

	db->unbind();
	this->unbind();

	int tmp;
	if( tmp=glGetError() ){
		std::cout<<"ERROR Texture::getData\n";
		std::cout<<gluErrorString(tmp)<<"\n";
	}
}

void Texture::getData(DataBuffer *db, FrameBuffer *fb, GLuint type){
	if(type==0)
		type=this->type;

	//Attention qd type different du type interne, verif des tailles

	if(db->getSize()<this->getDataSize()){
		std::cerr<<"Texture::getData -> Error: unmatching sizes\n";
		return;
	}

	fb->attachTexture(GL_COLOR_ATTACHMENT0_EXT, this);
	fb->bind();

	db->bind(GL_PIXEL_PACK_BUFFER_EXT);

	float *data=new float[512*512*4];

	glReadPixels(0, 0, size.x, size.y, this->getFormat(), type, db->use());

	delete [] data;
	

	db->unbind();
	fb->unbind();

	int tmp;
	if( tmp=glGetError() ){
		std::cout<<"ERROR Texture::getData\n";
		std::cout<<gluErrorString(tmp)<<"\n";
	}
}

void Texture::setData(DataBuffer *db, GLuint type) {
	if(type==0)
		type=this->type;

	this->bind();

	db->bind(GL_PIXEL_UNPACK_BUFFER_EXT);

	if(this->target==GL_TEXTURE_3D){
		glTexSubImage3D( this->target, 0, 0,0,0, this->size.x, this->size.y, this->size.z, this->format, type, db->use());
	}else{
		glTexSubImage2D( this->target, 0, 0,0, this->size.x, this->size.y, this->format, type, db->use());
	}

	db->unbind();
	this->unbind();

	int tmp;
	if( tmp=glGetError() ){
		std::cout<<"ERROR Texture::setData\n";
		std::cout<<gluErrorString(tmp)<<"\n";
	}
}

void Texture::getData(void *d, GLuint type) const{
	if(type==0)
		type=this->type;

	//Attention pas de verification de taille

	this->bind();

	glGetTexImage(this->getTarget(), 0, this->getFormat(), type, d);
	std::cout<<gluErrorString(glGetError())<<"!1 \n";

	this->unbind();
}

void Texture::setValue(const Vector2i &pos, const Vector4f &val){
	this->bind();
	glTexSubImage2D(this->target, 0, pos.x, pos.y, 1, 1, this->format, GL_FLOAT, &(val.x));
	//std::cout<<gluErrorString(glGetError())<<"\n";
	this->unbind();
}

//pour textures 2D seulement
int Texture::getOffset(const Vector2i &pos) const{
	//Test taille
	if(this->size.z>0)
		return -1;

	int off;
	off=pos.x*GLGetFormatSize(this->intFormat)+pos.y*GLGetFormatSize(this->intFormat)*this->size.x;

	return off;
}